#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#	Copyright (C) 2003-2010 Daniel Naber, Lutz Haseloff
#	Copyright (C) 2013-2015 Mechtilde Stehmann
#
#	Copyright (C) 2013-2015 for the gettext integration: Dr. Michael Stehmann
#	Copyright (C) 2016 Alexander Volkov <support@rusbitech.ru> (based on loook.py)
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

import os
import re
import sys
import string
import zipfile

class FileProcessor:
	def removeXMLMarkup(self, s, replace_with_space):
		s = re.compile("<!--.*?-->", re.DOTALL).sub('', s)
		repl = ''
		if replace_with_space:
			repl = ' '
		s = re.compile("<text:p [^>]*>", re.DOTALL).sub('\n', s)
		s = re.compile("</text:p>", re.DOTALL).sub('\n', s)
		s = re.compile("</w:p>", re.DOTALL).sub('\n', s)
		s = re.compile("<text:line-break/>", re.DOTALL).sub('\n', s)
		s = re.compile("<text:tab/>", re.DOTALL).sub('  ', s)
		
		s = re.compile("<[^>]*>", re.DOTALL).sub(repl, s)
		
		s = re.compile("&apos;", re.DOTALL).sub('\'', s)
		s = re.compile("&amp;", re.DOTALL).sub('&', s)
		s = re.compile("&quot;", re.DOTALL).sub('\"', s)
		s = re.compile("&gt;", re.DOTALL).sub('>', s)
		s = re.compile("&lt;", re.DOTALL).sub('<', s)
		return s

	def getSuffix(self, filename):
		suffix_match = re.compile(".*\.(.*)").match(filename)
		if suffix_match:
			suffix = suffix_match.group(1).lower()
		else:
			suffix = None
		return suffix

	def processFile(self, filename):
		suffix = self.getSuffix(filename)
		try:
			# Handle OpenOffice.org files:
			if suffix in ('sxw', 'stw',		# OOo   1.x swriter
					'sxc', 'stc',		# OOo   1.x scalc
					'sxi', 'sti',		# OOo   1.x simpress
					'sxg',				# OOo   1.x master document
					'sxm',				# OOo   1.x formula
					'sxd', 'std',		# OOo   1.x sdraw
					'odt', 'ott',		# OOo > 2.x swriter
					'odp', 'otp',		# OOo > 2.x simpress
					'odf',				# OOo > 2.x formula
					'odg', 'otg',		# OOo > 2.x sdraw
					'ods', 'ots',		# OOo > 2.x scalc
					):
				zip = zipfile.ZipFile(filename, "r")
				content = ""
				try:
					# search embedded files:
					filelist = zip.namelist()
					for filename in filelist:
						if filename.endswith("content.xml"):
							content += str(zip.read(filename), 'utf-8')

						if filename.endswith("document.xml"):
							content += str(zip.read(filename), 'utf-8')
							
					content = self.removeXMLMarkup(content, replace_with_space=0)
				except KeyError as err:
					print("Warning: %s not found in '%s'" % (err, filename))
					return None
				print(content)
				return

			# Handle MS-Office (>= 2007) files:
			if suffix in ('docx', 'dotx',		# MS-Word Documents >= 2007
				'xlsx', 'xltx',		# MS-Excel-Documents >= 2007
				):
				zip = zipfile.ZipFile(filename, "r")
				content = ""
				try:
					# search embedded files:
					filelist = zip.namelist()
					for filename in filelist:

						if filename.endswith("document.xml"):
							content += str(zip.read(filename), 'utf-8')

						if filename.endswith("sharedStrings.xml"):
							content += str(zip.read(filename), 'utf-8')

					content = self.removeXMLMarkup(content, replace_with_space=0)
				except KeyError as err:
					print("Warning: %s not found in '%s'" % (err, filename))
					return None
				print(content)
				return
			# Handle MS-Office (>= 2007) MS-PowerPoint files:
			if suffix in ('pptx', 		# MS-PowerPoint-Documents >= 2007
				):
				zip = zipfile.ZipFile(filename, "r")
				content = ""
				try:
					# search embedded files:
					filelist = zip.namelist()
					slideslist = []
					for element in filelist:
						if element[4:12]  == "slides/s":
							slideslist.append(element)
					content =""
					for element in slideslist:
						content += str(zip.read(element), 'utf-8')

					content = self.removeXMLMarkup(content, replace_with_space=0)
				except KeyError as err:
					print("Warning: %s not found in '%s'" % (err, filename))
					return None
				print(content)
				return

		except zipfile.BadZipfile as err:
			pass
		except IOError as err:
			pass
		return None

fileProcessor = FileProcessor()
fileProcessor.processFile(sys.argv[1])
